#ifndef CLASS_TEXTURE
#define CLASS_TEXTURE

#include <stdexcept>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glext.h>

/// Texture class
class Texture
{
	private:
		GLuint _id;
		void Load(std::string filename) throw(std::invalid_argument);
		void Load(SDL_Surface *image) throw(std::invalid_argument);
		void Load(GLenum format, int width, int height, GLfloat *pixels);
		void Load(GLenum iformat, int width, int height, GLubyte *pixels);
	public:
		Texture();
		Texture(std::string filename, int unit = -1);
		Texture(SDL_Surface *surface, int unit = -1);
		Texture(GLenum format, int width, int height, GLfloat *pixels, int unit = -1);
		Texture(GLenum format, int width, int height, GLubyte *pixels, int unit = -1);
		~Texture();
		/// Set texture filtering for both magnification and minification
		void SetFiltering(GLenum filter);
		/// Bind the texture to a specific texture unit
		void Bind(int unit = 0);
		/// Disable texture rendering of a specific unit
		static void Unbind(int unit = 0);
};

#endif
